title: The Inner Osborne Effect
url: http://raganwald.com/2021/10/28/the-inner-osborne-effect.html
hash_url: a98225895499217f24a9e081c2684dc4

<p>In software development, we talk a lot about software anti-patterns, how to recognize them, and how to extricate yourself from them via refactoring.</p>

<blockquote>
  <p>An <strong>anti-pattern</strong> is a common response to a recurring problem that is usually ineffective and risks being highly counterproductive. The term, coined in 1995 by computer programmer <a href="https://en.wikipedia.org/wiki/Andrew_Koenig_(programmer)">Andrew Koening</a>, was inspired by the book <a href="https://en.wikipedia.org/wiki/Design_Patterns_(book)">Design Patterns</a>, which highlights a number of design patterns in software development that its authors considered to be highly reliable and effective.</p>
</blockquote>

<blockquote>
  <p>The term was popularized three years later by the book <a href="https://en.wikipedia.org/wiki/AntiPatterns">AntiPatterns</a>, which extended its use beyond the field of software design to refer informally to any commonly reinvented but bad solution to a problem.</p>
</blockquote>

<p>Anti-patterns occur throughout businesses, not just in software code and architecture. One famous business anti-pattern is the <a href="https://en.wikipedia.org/wiki/Osborne_effect">Osborne Effect</a>, which is when a company pre-announces a future product that will obsolete the product line it’s selling today.</p>

<p>Customers take notice and cancel their orders for the current product, waiting for the wonderful future in the company’s marketing brochure. This stalls the company’s momentum, kills the company’s revenue, and in extreme cases, drives it right out of business.</p>

<p>But the Osborne Effect isn’t just a marketing and sales anti-pattern. It’s also a product management anti-pattern, the <strong>Inner Osborne Effect</strong>.</p>

<hr>

<center><img src="/assets/images/retro-futurism-city.jpg"></center>

<hr>

<h1 id="the-product-of-tomorrow">The Product of Tomorrow</h1>

<p>I worked for two different companies who inflicted the Inner Osborne Effect on themselves, and both did so in theatrically dramatic ways. Pull up a pew, I’ll describe what happened, and how it became a disaster. The following story is a mashup inspired by both sets of circumstances, with some artistic license taken.</p>

<p>Our semi-fictitious company began its life as a scrappy startup selling into mid- to enterprise-sized customers. It found early success, and powered by its sales-led revenues, grew to dominate its niche. Driven by closing deals, its long-term development was constantly derailed by “fire drills” to ship new features that were alleged to be keys to closing big contracts, or by bug fixes driven by whichever customer complained the loudest at contract renewal time.</p>

<p>Tech debt piled on faster than pounds at an all-you-can-eat pancake breakfast. Development velocity slowed, which increased the urgency of shipping features at the expense of writing quality code or refactoring existing problem code.</p>

<p>But there was another threat looming:</p>

<p>Because of the company’s success, each sale was paradoxically getting harder to make. It had already plucked the low-hanging fruit in its niche, so the remaining customers were those who had less of a fit for the company’s product. And each new feature generated less net new revenue than the preceding features, because with fewer customers remaining in the niche, each new feature unlocked a proportionally smaller share of net new revenue.</p>

<p>If the company was to continue to grow, It needed to escape its niche with a bold new effort.</p>

<hr>

<center><img src="/assets/images/vp-marketing.jpg"></center>

<hr>

<h2 id="lumburgh">Lumburgh</h2>

<p>By this time, trust between the C-Suite and Engineering was at an all-time low. The executives didn’t understand why bugs were climbing and velocity was dropping. The <a href="http://raganwald.com">Director of Engineering</a> was articulate in explaining exactly what was going on, but Lumburgh, the VP of Marketing, would always interject with “Poppycock! Back when I worked at FamousCo, we had none of these problems!”</p>

<p>The C-Suite were receptive when Lumburgh approached them on the sly. “I can start a skunkworks, remotely, under my direct command, to build <strong>The Product of Tomorrow</strong>. It will be <a href="https://www.youtube.com/watch?v=wPO8PqHGWFU">both a floor wax <em>and</em> a dessert topping</a>!”</p>

<p>Engineering was strictly fire-walled off from The Product of Tomorrow Team, which would emerge from stealth RealSoonNow™️ to replace everything Engineering was working on today.</p>

<p>How did the engineers feel about this? They were already demoralized by the feature factory mindset. They had been complaining forever that if they weren’t given the time to build the product right, they’d never find the time to build it over, and now they felt punished for management’s choice to ignore their warnings.</p>

<p>Resignations followed, first a trickle, then a flow.</p>

<p>Meanwhile, the company’s Product Management group was in chaos. Every time they wanted to build something, they’d be stopped with “Hold up, that will be half the cost and twice the value with the Product of Tomorrow!”</p>

<p>The current product was eventually put on life support, because senior management had neither the will, nor the resources, to do anything except fix bugs while waiting for the Product of Tomorrow.</p>

<hr>

<center><img src="/assets/images/wizard-of-oz.jpg"></center>

<hr>

<h2 id="whither-the-product-of-tomorrow">Whither The Product of Tomorrow?</h2>

<p>I was long gone by the time the Product of Tomorrow was killed. Yes, the Product of Tomorrow failed, and the Inner Osborne Effect took the existing product with it. The Product of Tomorrow turned out to be demo-ware, carefully crafted to look good in front of the CEO, but lacked essential features “under the hood.”</p>

<p>The central cause of the failure was another anti-pattern, <strong>CEO as Customer</strong>. And it turns out that a company that can’t invest in the quality of its mainline product, won’t invest in the quality of its replacement, either. That’s a management anti-pattern, and starting over with new people and a blank piece of paper doesn’t change management. Only management can change management.</p>

<hr>

<center><img src="/assets/images/messagepad-100.jpg"></center>

<hr>

<h2 id="from-osborne-1-to-messagepad-100">From Osborne 1 to MessagePad 100</h2>

<p>But getting back to our company and its executives, the problem wasn’t that the executives bet on the Product of Tomorrow, but that they also <em>announced to its Engineering and Product Management groups that they were betting everything on the Product of Tomorrow</em>. And then they followed through by obstructing any attempt by Engineering or Product Management to invest in the original product.</p>

<p>Compare and contrast this approach with Apple, who having succeeded with Apple II, went on to make big bets on Apple III, Lisa, Macintosh, Newton, iPhone, and iPad.</p>

<p>Apple III and Lisa failed. Macintosh was underpowered and overpriced on launch. But Apple continued to invest in Apple II, which financed investing in Macintosh, which became its future. Macintosh financed betting on Newton, which failed, and iPhone, which succeeded. Now Apple’s betting on AppleTV, iPad and Apple Watch, which are “nice little businesses” compared to iPhone. iPad Pro is close to displacing Macintosh, but Apple is famouly tight-lipped about its visions of the future, and only <a href="https://en.wikipedia.org/wiki/Taligent#Development">once</a>—to my knowledge—made the grevious mistake of inflicting the Inner Osborne Effect on itself.</p>

<p>Could the Product of Tomorrow have succeeded? Maybe. But following Apple’s example, the way to do that would have been to make it fully independent, and continue to build the existing product as if the plans for the Product of Tomorrow did not exist. Call the Product of Tomorrow “research.” Or an “experiment.” Locate its team in <a href="https://www.folklore.org/StoryView.py?story=Texaco_Towers.txt">Texaco Towers</a>. But don’t call it the future that will obsolete the present. And especially don’t respond to that by choking all progress on the cash cow that is financing the company’s bets.</p>

<h3 id="post-scriptum">post scriptum</h3>

<p><em>The Obsborne Effect was not the sole cause of the failure of the Osborne Computer Company, and the Inner Osborne Effect I described above wasn’t the sole cause of either source company’s failures. But both companies would have had a fighting chance to succeed had they not pre-announced that they were going all-in on an all-singing, all-dancing piece of vapourware built outside of their core product development groups.</em></p>
